"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const messaging_1 = require("@nteract/messaging");
/**
 * Class used by widgets to communicate with the backend
 */
class WidgetComm {
    /**
     *
     * @param comm_id uuid
     * @param target_name comm handler
     * @param target_module used to select a module that is responsible for handling the target_name
     * @param kernel A reference to the
     */
    constructor(comm_id, target_name, target_module, kernel) {
        this.comm_id = comm_id;
        this.target_name = target_name;
        this.target_module = target_module;
        this.kernel = kernel;
    }
    /**
     * Opens a sibling comm in the backend
     * @param  data
     * @param  callbacks
     * @param  metadata
     * @return msg id
     */
    open(data, callbacks, metadata, buffers) {
        const message = messaging_1.createCommOpenMessage(this.comm_id, this.target_name, this.flattenBufferArrays(buffers), this.target_module);
        this.kernel.channels.next(message);
        this.hookupReplyCallbacks(message, callbacks);
        return message.header.msg_id;
    }
    /**
     * Sends a message to the sibling comm in the backend
     * @param  data
     * @param  callbacks
     * @param  metadata
     * @param  buffers
     * @return message id
     */
    send(data, callbacks, metadata, buffers) {
        const message = messaging_1.createCommMessage(this.comm_id, data, this.flattenBufferArrays(buffers));
        this.kernel.channels.next(message);
        this.hookupReplyCallbacks(message, callbacks);
        return message.header.msg_id;
    }
    /**
     * Closes the sibling comm in the backend
     * @param  data
     * @param  callbacks
     * @param  metadata
     * @return msg id
     */
    close(data, callbacks, metadata, buffers) {
        throw "close not yet implemented!";
    }
    /**
     * Register a message handler
     * @param  callback, which is given a message
     */
    on_msg(callback) {
        this.kernel.channels
            .pipe(messaging_1.ofMessageType("comm_msg"), messaging_1.withCommId(this.comm_id))
            .subscribe((msg) => {
            callback(msg);
        });
    }
    /**
     * Register a handler for when the comm is closed by the backend
     * @param  callback, which is given a message
     */
    on_close(callback) {
        this.kernel.channels
            .pipe(messaging_1.ofMessageType("comm_close"), messaging_1.withCommId(this.comm_id))
            .subscribe((msg) => {
            callback(msg);
        });
    }
    /**
     * Convert the buffer array that iPyWidgets use into the single
     * Uint8Array that nteract expects for creating messages
     * @param buffers iPyWidgets buffer format
     * @return nteract buffer format
     */
    flattenBufferArrays(buffers) {
        if (buffers === undefined)
            return undefined;
        // determining size and creating array
        let byteLength = 0;
        for (let b of buffers) {
            byteLength += b.byteLength;
        }
        let flattened = new Uint8Array(byteLength);
        // copying buffers over
        for (let b of buffers) {
            let arr = b instanceof ArrayBuffer ? new Uint8Array(b) : b;
            flattened.set(arr);
        }
        return flattened;
    }
    /**
     * Subscribes to message replies and routes them to the right callbacks
     * provided by widgets
     * @param message
     * @param callbacks
     */
    hookupReplyCallbacks(message, callbacks) {
        this.kernel.channels.pipe(messaging_1.childOf(message)).subscribe((reply) => {
            if (reply.channel == "shell" &&
                callbacks.shell &&
                callbacks.shell.reply) {
                callbacks.shell.reply(reply);
            }
            else if (reply.channel == "stdin" && callbacks.input) {
                callbacks.input(reply);
            }
            else if (reply.channel == "iopub" && callbacks.iopub) {
                if (callbacks.iopub.status && reply.header.msg_type === "status") {
                    callbacks.iopub.status(reply);
                }
                else if (callbacks.iopub.clear_output &&
                    reply.header.msg_type === "clear_output") {
                    callbacks.iopub.clear_output(reply);
                }
                else if (callbacks.iopub.output) {
                    switch (reply.header.msg_type) {
                        case "display_data":
                        case "execute_result":
                        case "stream":
                        case "error":
                            callbacks.iopub.output(reply);
                            break;
                        default:
                            break;
                    }
                }
            }
        });
    }
}
exports.WidgetComm = WidgetComm;
